#!/usr/bin/python3

import argparse
import os
import json
import shutil
import subprocess

parser = argparse.ArgumentParser(description='Mount a Tronius package to a specified location.')
parser.add_argument('package', help='Tronius package name')
parser.add_argument('mountpoint', help='The directory where to mount the contents of the package.')
parser.add_argument('--core', action="store_true", help='If true, tread this package as the core package.')

args = parser.parse_args()

backup_file = "/var/IGP/backup.chk"
packages_dir = "/var/IGP/packages"
jurisdiction_file = "/var/IGP/machine-jurisdiction"
dependencies_file = "dependencies.json"
manifest_file = "manifest.json"

backupChk = None
try:
	with open(backup_file) as f:
		backupChk = json.load(f)
except:
	pass

if (backupChk is not None) and os.path.exists(jurisdiction_file):
	jurisdiction = ""
	with open(jurisdiction_file, 'r') as f:
		jurisdiction = f.read().rstrip()
	backupChk = backupChk[jurisdiction]

def get_pack_prefix(pack) -> str:
	if (backupChk is not None) and (pack in backupChk):
		return ".old."
	else:
		return ""

def mount_pack(pack_name, mountpoint, is_core = False) -> int:
	source_file = os.path.join(packages_dir, get_pack_prefix(pack_name) + pack_name + ".pack")
	if not os.path.exists(source_file):
		print("Package file at %s could not be found." % source_file)
		exit(1)

	mount_result = -1
	if is_core:
		rw_core_dir = "/var/IGP/rwcore"

		rw_mount_mode = 0
		if os.path.exists("/var/IGP/.mount_core_rw"):
			rw_mount_mode = 1
		elif os.path.exists("/var/IGP/.mount_core_rw_nopersist"):
			rw_mount_mode = 2

		if rw_mount_mode != 1:
			shutil.rmtree(rw_core_dir, True) # remove previouslly mounted RW core if it exists
		
		if rw_mount_mode == 0:
			# remove a possible symbolic link at /home/IGP to /var/IGP/rwcore
			if os.path.exists(mountpoint) and os.path.islink(mountpoint):
				os.remove(mountpoint)
			# create a folder at /home/IGP that we will mount the core pack to
			os.makedirs(mountpoint, 551, True)
			mount_result = os.system("mount -t squashfs -o loop %s %s" % (source_file, mountpoint))
		else:
			# if rw core path doesn't exist, create it and copy core pack content into it
			if not os.path.exists(rw_core_dir):
					# remove the directory /home/IGP which is used as a mount target
					shutil.rmtree(mountpoint, True)
					# create a symbolic link at /home/IGP that points to /var/IGP/rwcore
					os.symlink(rw_core_dir, mountpoint)
					# copy the contents of the core pack into the RW core folder
					mount_result = os.system("unsquashfs -d %s %s" % (rw_core_dir, source_file))
			else:
				mount_result = 0
	else:
		os.makedirs(mountpoint, 551, True)
		mount_result = os.system("mount -t squashfs -o loop %s %s" % (source_file, mountpoint))

	if mount_result != 0:
		print("IGP-mount: ERROR occoured while mounting '%s' on %s - error code %s." % (pack_name, mountpoint, mount_result))
		exit(2)

	deps = None
	try:
		with open(os.path.join(mountpoint, dependencies_file)) as f:
			deps = json.load(f)
	except:
		pass

	if isinstance(deps, list):
		for dependency in deps:
			if "packageName" in dependency:
				dependency_pack_name = dependency["packageName"]
				dependency_result = mount_pack(dependency_pack_name, os.path.join("/var/IGP/addons", dependency_pack_name))
				if dependency_result == 0:
					print("Successfully mounted dependency '%s'." % dependency_pack_name)
				else:
					print("ERROR: Could not mount dependency '%s' - error code %d" % (dependency_pack_name, dependency_result))
					mount_result = 7

	if mount_result != 0:
		print("IGP-mount: ERROR occoured while mounting dependencies of '%s'." % pack_name)
		return mount_result
	
	print("IGP-mount: '%s' mounted on %s." % (pack_name, args.mountpoint))

	#load package if loader exists
	package_init_script = os.path.join(mountpoint, "pack-init.sh")
	if os.path.exists(package_init_script):
		firstrun_file = ""

		manifestJson = None
		try:
			with open(os.path.join(mountpoint, manifest_file)) as f:
				manifestJson = json.load(f)
		except:
			pass

		if manifestJson is not None:
			firstrun_file = "/var/IGP/.firstrun_%s_%d.%d.%d" % (pack_name, manifestJson["majorVersion"], manifestJson["minorVersion"], manifestJson["patchVersion"])
		
		environment = os.environ.copy()
		environment["IS_FIRSTRUN"] = "0" if (firstrun_file and os.path.exists(firstrun_file)) else "1"

		pack_init_result = subprocess.call([package_init_script], cwd=mountpoint, env=environment, shell=True)
		
		if firstrun_file:
			with open(firstrun_file, "w") as f:
				f.write("%s was already initialized. Delete this file to patch again with pack-init.sh.")

		if pack_init_result == 0:
			print("IGP-mount: '%s' loaded." % pack_name)
			os.system("notify-send \"IGP-mount: '%s' loaded.\"" % pack_name)
		else:
			print("IGP-mount: ERROR loading '%s' pack." % pack_name)
			os.system("notify-send \"IGP-mount: ERROR loading '%s' pack.\"" % pack_name)
		
		return pack_init_result

	return mount_result

exit(mount_pack(args.package, args.mountpoint, args.core))
